import numpy as np

def segmentor(tokens, segment_num):

    ids = np.rint(np.linspace(0, len(tokens), segment_num, endpoint = False)).astype(int)
    segments_id = zip(ids, np.append(ids[1:], len(tokens)))

    segments_token = [tokens [start: end] for start, end in segments_id]
    segments_text = [' '.join(tokens) for tokens in segments_token]

    return segments_text, segments_token, segments_id





from sentence_transformers import SentenceTransformer

class Embedding():
    def __init__(self):
        self.sentence_encode =  SentenceTransformer('sentence-transformers/multi-qa-MiniLM-L6-cos-v1')

    def sentence_embedding(self, text):
        return self.sentence_encode.encode(text)






from transformers import AutoTokenizer
from transformers import AutoModelForSeq2SeqLM
from parrot.filters import Adequacy
from parrot.filters import Fluency
from parrot.filters import Diversity

access_token = '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'

class Paraphrase_Generation():
  
    def __init__(self, model_tag="prithivida/parrot_paraphraser_on_T5", use_gpu=False):
        self.model_tag = model_tag
        self.use_gpu = use_gpu

    def load_model(self):
        print('😊 Parrot is loading on your system...\n'+'-'*50)

        self.tokenizer = AutoTokenizer.from_pretrained(self.model_tag, cache_dir = '../../../SavedModels/parrotT5', use_auth_token=access_token)
        self.model     = AutoModelForSeq2SeqLM.from_pretrained(self.model_tag, cache_dir = '../../../SavedModels/parrotT5', use_auth_token=access_token)
        self.adequacy_score = Adequacy()
        self.fluency_score  = Fluency()
        self.diversity_score= Diversity()


    def infer(self, input_phrase, diversity_ranker="levenshtein", do_diverse=False, max_return_phrases = 10, max_length=32, adequacy_threshold = 0.90, fluency_threshold = 0.90):
        print(f'😊 Parrot is infering : {input_phrase} ...\n'+'-'*50)

        if self.use_gpu:
            device= "cuda:0"
        else:
            device = "cpu"

        self.model     = self.model.to(device)

        import re

        save_phrase = input_phrase
        if len(input_phrase) >= max_length:
            max_length += 32	
			
        input_phrase = re.sub('[^a-zA-Z0-9 \?\'\-\/\:\.]', '', input_phrase)
        input_phrase = "paraphrase: " + input_phrase
        input_ids = self.tokenizer.encode(input_phrase, return_tensors='pt')
        input_ids = input_ids.to(device)

        if do_diverse:
            for n in range(2, 9):
                if max_return_phrases % n == 0:
                    break 
                #print("max_return_phrases - ", max_return_phrases , " and beam groups -", n)            
                preds = self.model.generate(
                input_ids,
                do_sample=False, 
                max_length=max_length, 
                num_beams = max_return_phrases,
                num_beam_groups = n,
                diversity_penalty = 2.0,
                early_stopping=True,
                num_return_sequences=max_return_phrases)
        else: 
            preds = self.model.generate(
                input_ids,
                do_sample=True, 
                max_length=max_length, 
                top_k=50, 
                top_p=0.95, 
                early_stopping=True,
                num_return_sequences=max_return_phrases) 
        

        paraphrases= set()

        for pred in preds:
            gen_pp = self.tokenizer.decode(pred, skip_special_tokens=True).lower()
            gen_pp = re.sub('[^a-zA-Z0-9 \?\'\-]', '', gen_pp)
            paraphrases.add(gen_pp)


        adequacy_filtered_phrases = self.adequacy_score.filter(input_phrase, paraphrases, adequacy_threshold, device )
        if len(adequacy_filtered_phrases) > 0 :
            fluency_filtered_phrases = self.fluency_score.filter(adequacy_filtered_phrases, fluency_threshold, device )
            if len(fluency_filtered_phrases) > 0 :
                diversity_scored_phrases = self.diversity_score.rank(input_phrase, fluency_filtered_phrases, diversity_ranker)
                para_phrases = []
                for para_phrase, diversity_score in diversity_scored_phrases.items():
                    para_phrases.append((para_phrase, diversity_score))
                para_phrases.sort(key=lambda x:x[1], reverse=True)
                return para_phrases
            else:
                return [(save_phrase,0)]


from parsivar import Normalizer, Tokenizer
from embedding import Embedding
from text_segmentation import segmentor
import numpy as np
class KeywordExtraction():
    def __init__(self):
        pass
    
    def load_model(self):
        self.normalizer = Normalizer()
        self.tokenizer = Tokenizer()
        self.embeder = Embedding()
    
    def preprocess(self, text):
        normal_text = self.normalizer.normalize(text)
        return normal_text

    def train(self):
        pass
    
    def _semantic_score_up(self, scores):
        return np.dot(scores, self.weights)

    def infer(self, text, segment_num= 2, top_n= 5):
        tokens = self.tokenizer.tokenize_words(text)
        words_type = list(set(tokens))

        text_embedding = self.embeder.sentence_embedding([text])
        segments_embedding = self.embeder.sentence_embedding(segmentor(tokens, segment_num)[0])
        words_type_embedding = self.embeder.sentence_embedding(words_type)

        words_text_sim = np.matmul(text_embedding, words_type_embedding.T)
        words_segments_sim = np.matmul(segments_embedding, words_type_embedding.T)

        similar_matrix = np.concatenate((words_text_sim, words_segments_sim), axis = 0)

        self.weights = np.ones(similar_matrix.shape[0])
        self.weights[0] *= 3 

        word_score = zip(words_type, np.apply_along_axis(self._semantic_score_up, 0, similar_matrix))

        sorted_word_score = sorted(dict(word_score).items(), key=lambda y: y[1], reverse=True)

        return sorted_word_score[:top_n+1]


    def postprocess():
        pass


#!/usr/bin/env python3

from vosk import Model, KaldiRecognizer, SetLogLevel
import os
import wave

wf = wave.open("test.wav", "rb")
model = Model("..")
rec = KaldiRecognizer(model, wf.getframerate())

while True:
    data = wf.readframes(4000)
    if len(data) == 0:
        break
    if rec.AcceptWaveform(data):
        print(rec.Result())
    else:
        print(rec.PartialResult())

print(rec.FinalResult())



import datetime


def log_it(object):
    def wrapper(*args,**kwargs):
        
        dts = datetime.datetime.now()
        print(f"\nAt {dts}, 😊VoXera is here: {object.__name__} - {args} - {kwargs}")
        result = object(*args,**kwargs)
        dts = datetime.datetime.now()
        print(f"At {dts}, 😊VoXera has left from: {object.__name__} - {args} - {kwargs}")

        return result
    return wrapper








import warnings
from VXparrot import Paraphrase_Generation
warnings.filterwarnings("ignore")

''' 
uncomment to get reproducable paraphrase generations
def random_state(seed):
  torch.manual_seed(seed)
  if torch.cuda.is_available():
    torch.cuda.manual_seed_all(seed)

random_state(1234)
'''

#Init models (make sure you init ONLY once if you integrate this to your code)
parrot = Paraphrase_Generation()

parrot.load_model()

phrases = ["Natural language processing and representation learning in the text and audio domains are of interest to me.",
          "Can you recommed some upscale restaurants in Newyork?",
           "What are the famous places we should not miss in Russia?"
]

for phrase in phrases:
  paraphrases = parrot.infer(input_phrase=phrase)
  for paraphrase in paraphrases:
    print(paraphrase)

"""
test output:
😊 Parrot is loading on your system...
--------------------------------------------------
2022-10-19 13:43:16.555966: W tensorflow/stream_executor/platform/default/dso_loader.cc:64] Could not load 
dynamic library 'cudart64_110.dll'; dlerror: cudart64_110.dll not found
2022-10-19 13:43:16.561065: I tensorflow/stream_executor/cuda/cudart_stub.cc:29] Ignore above cudart dlerror if you do not have a GPU set up on your machine.
😊 Parrot is infering : Natural language processing and representation learning in the text and audio domains are of interest to me. ...
--------------------------------------------------
('natural language processing and representation learning are of interest to me in the text and audio domains', 54)
😊 Parrot is infering : Can you recommed some upscale restaurants in Newyork? ...
--------------------------------------------------
('recommend some good places to eat in new york city?', 42)
('list some of the best restaurants in new york?', 33)
('recommend some of the best restaurants in newyork?', 31)
('can you suggest some upscale restaurants in new york?', 21)
('can you recommend some good restaurants in new york?', 21)
('can you recommend some of the upscale restaurants in newyork?', 20)
('can you recommend upscale restaurants in new york?', 19)
('can you recommend some upscale restaurants in new york?', 14)
('can you recommend some upscale restaurants in newyork?', 13)
😊 Parrot is infering : What are the famous places we should not miss in Russia? ...
--------------------------------------------------
("what are some of the most famous places in russia that you'd never miss?", 45)
('recommend some of the must-see places in russia?', 41)
('list some of the most popular places to visit in russia?', 40)
('list the best places to visit in russia?', 39)
('recommend some of the top places to visit in russia?', 37)
('list the most famous places to visit in russia?', 34)
"""


from VXperdeepke import KeywordExtraction

text = "بر اساس تحلیل نقشه‌های همدیدی و آینده‌نگری سازمان هواشناسی امروز در استان‌های ساحلی دریای خزر، اردبیل، شمال آذربایجان شرقی و ارتفاعات البرز مرکزی بارش باران، همراه با وزش باد شدید موقتی و کاهش نسبی دما پیش‌بینی شده است. فردا از میزان بارش‌های این مناطق کاسته شده و فقط در سواحل شمالی بارش پراکنده روی می‌دهد."
segment_num = 3
top_n = 5

ke = KeywordExtraction()
ke.load_model()
word_score = ke.infer(text, segment_num= segment_num, top_n= top_n)

print(word_score)

"""
test output:

[('آینده\u200cنگری', 4.234069764614105), ('استان\u200cهای', 4.143936276435852), ('بارش\u200cهای', 4.121438443660736), ('نقشه\u200cهای', 4.015317618846893), ('دریای', 3.9617764949798584), ('شمالی', 3.9322071075439453)] 
"""


import torch
from transformers import PegasusForConditionalGeneration, PegasusTokenizer
from VoXera.Logging.VXlogging import log_it

torch_device = 'cuda' if torch.cuda.is_available() else 'cpu'




class Paraphrase_Generation():
    def __init__(self, model_tag='tuner007/pegasus_paraphrase', use_gpu=False):
        self.model_tag = model_tag

    @log_it
    def load_model(self):
        cache_dir = '../../../SavedModels/pegasus'

        self.tokenizer = PegasusTokenizer.from_pretrained(self.model_tag, cache_dir=cache_dir)
        self.model = PegasusForConditionalGeneration.from_pretrained(self.model_tag, cache_dir= cache_dir).to(torch_device)

    @log_it
    def infer(self, text, num_return_sequences=10, num_beams=10):
        batch = self.tokenizer([text],truncation=True,padding='longest',max_length=60, return_tensors="pt").to(torch_device)
        translated = self.model.generate(**batch,max_length=60,num_beams=num_beams, num_return_sequences=num_return_sequences, temperature=1.5)
        tgt_text = self.tokenizer.batch_decode(translated, skip_special_tokens=True)
        
        return tgt_text





from VXpegasus import Paraphrase_Generation

context = "The ultimate test of your knowledge is your capacity to convey it to another."

pegasus = Paraphrase_Generation()
pegasus.load_model()

print(pegasus.infer(text= context))


from VXvosk import AutomaticSpeechRecogniton

model_name = 'small'
model_dir = "../../SavedModels/Vosk/"
my_path = "../../TestData/news_1.wav"

asr = AutomaticSpeechRecogniton()

asr.load_model(model_dir= model_dir, model_name= model_name)
text, segments= asr.infer(speech_file_path= my_path)


print('Transcribed Text:', text)
print('Speech Segments',segments)

"""
test output:

😊 Vosk is loading on your system...
--------------------------------------------------
😊 Vosk is decoding : ../Test_Data/news_1.wav ...
----------------------------------------------------------------------------------------------------
Transcribed Text: رئیسه سازمان نظام دامپزشکی از برگزاری چهارمین دوره شوراهای استانی نظام دامپزشکی در روز جمعه خبر داد
Speech Segments [{'conf': 0.529013, 'end': 0.9, 'start': 0.3, 'word': 'رئیسه'}, {'conf': 1.0, 'end': 1.47, 'start': 0.96, 'word': 'سازمان'}, {'conf': 1.0, 'end': 1.98, 'start': 1.47, 'word': 'نظام'}, {'conf': 1.0, 'end': 2.97, 'start': 2.01, 'word': 'دامپزشکی'}, {'conf': 1.0, 'end': 3.15, 'start': 2.97, 'word': 'از'}, {'conf': 1.0, 'end': 3.72, 'start': 3.15, 'word': 'برگزاری'}, {'conf': 1.0, 'end': 4.2, 'start': 3.72, 'word': 'چهارمین'}, {'conf': 1.0, 'end': 4.62, 'start': 4.2, 'word': 'دوره'}, {'conf': 1.0, 'end': 5.43, 'start': 4.86, 'word': 'شوراهای'}, {'conf': 1.0, 'end': 5.88, 'start': 5.46, 'word': 'استانی'}, {'conf': 1.0, 'end': 6.27, 'start': 5.91, 'word': 'نظام'}, {'conf': 1.0, 'end': 7.02, 'start': 6.3, 'word': 'دامپزشکی'}, {'conf': 1.0, 'end': 7.38, 'start': 7.2, 'word': 'در'}, {'conf': 1.0, 'end': 7.62, 'start': 7.38, 'word': 'روز'}, {'conf': 1.0, 'end': 7.95, 'start': 7.62, 'word': 'جمعه'}, {'conf': 1.0, 'end': 8.34, 'start': 7.95, 'word': 'خبر'}, {'conf': 1.0, 'end': 8.46, 'start': 8.34, 'word': 'داد'}]
"""


import whisper

class AutomaticSpeechRecogniton():
    def __init__(self):
        pass

    def load_model(self, model_dir= "../SavedModels/whisper/", model_name= 'base'):
        """
        Parameters:
        ----------
        1. model_dir: parent directory of your model
        2. model_name: name of your model (name of folder or binary file)
        """
        print('😊 Whisper is loading on your system...\n'+'-'*50)
        self.model = whisper.load_model(name= model_name,
                                download_root= model_dir)
    def preprocess(self):
        pass

    def infer(self, speech_file_path):
        """
        Parameters:
        ----------
        1. speech_file_path: audio address which you want to transcribe

        Returns:
        --------
        1. text = transcribed your speech file as an string
        2. segments = speech signal segmented by (start, end, content)
        """
        print(f'😊 Whisper is decoding : {speech_file_path} ...\n'+'-'*50)

        options = {"language": 'fa', "task": "transcribe"}
        result = self.model.transcribe(audio= speech_file_path,
                                verbose= None, **options)
        text = result["text"]
        segments = result["segments"]

        return text, segments
    
    def postprocess(self):
        pass


import wave
from vosk import Model, KaldiRecognizer, SetLogLevel
import ast
from VoXera.Logging.VXlogging import log_it

class AutomaticSpeechRecogniton():
    def __init__(self):
        pass
    
    @log_it
    def load_model(self, model_dir= "../SavedModels/Vosk/", model_name= 'small'):
        """
        Parameters:
        ----------
        1. model_dir: parent directory of your model
        2. model_name: name of your model (name of folder or binary file)
        """
        SetLogLevel(-1)

        self.model = Model(model_path= model_dir + model_name)

    @log_it
    def preprocess(self):
        pass
    
    @log_it
    def infer(self, speech_file_path):
        """
        Parameters:
        ----------
        1. speech_file_path: audio address which you want to transcribe

        Returns:
        --------
        1. text = transcribed your speech file as an string
        2. segments = speech signal segmented by (start, end, content, confidence)
        """

        speech_signal = wave.open(speech_file_path, 'rb')

        rec = KaldiRecognizer(self.model, speech_signal.getframerate())
        rec.SetWords(True)
        rec.SetPartialWords(True)
        
        while True:
            data = speech_signal.readframes(4000)
            if len(data) == 0:
                break
            if rec.AcceptWaveform(data):
                rec.Result()
            else:
                rec.PartialResult()

        result =ast.literal_eval(rec.FinalResult())

        segments = result['result']
        text = result['text']
        
        return text, segments
    
    @log_it
    def postprocess(self):
        pass